// Copyright (C) 2024 UOS Technology Co., Ltd.
// SPDX-FileCopyrightText: 2024 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: GPL-3.0-or-later

#include "utils/string_helper.h"

ANYTHING_NAMESPACE_BEGIN

ANYTHING_NAMESPACE_END