/****************************************************************************
** Meta object code from reading C++ file 'base_event_handler.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../deepin-anything/src/server/include/core/base_event_handler.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'base_event_handler.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_base_event_handler_t {
    QByteArrayData data[42];
    char stringdata0[521];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_base_event_handler_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_base_event_handler_t qt_meta_stringdata_base_event_handler = {
    {
QT_MOC_LITERAL(0, 0, 18), // "base_event_handler"
QT_MOC_LITERAL(1, 19, 15), // "D-Bus Interface"
QT_MOC_LITERAL(2, 35, 19), // "com.deepin.anything"
QT_MOC_LITERAL(3, 55, 24), // "autoIndexInternalChanged"
QT_MOC_LITERAL(4, 80, 0), // ""
QT_MOC_LITERAL(5, 81, 17), // "autoIndexInternal"
QT_MOC_LITERAL(6, 99, 24), // "autoIndexExternalChanged"
QT_MOC_LITERAL(7, 124, 17), // "autoIndexExternal"
QT_MOC_LITERAL(8, 142, 20), // "asyncSearchCompleted"
QT_MOC_LITERAL(9, 163, 7), // "results"
QT_MOC_LITERAL(10, 171, 6), // "search"
QT_MOC_LITERAL(11, 178, 4), // "path"
QT_MOC_LITERAL(12, 183, 8), // "keywords"
QT_MOC_LITERAL(13, 192, 6), // "offset"
QT_MOC_LITERAL(14, 199, 9), // "max_count"
QT_MOC_LITERAL(15, 209, 4), // "type"
QT_MOC_LITERAL(16, 214, 5), // "after"
QT_MOC_LITERAL(17, 220, 6), // "before"
QT_MOC_LITERAL(18, 227, 18), // "traverse_directory"
QT_MOC_LITERAL(19, 246, 10), // "removePath"
QT_MOC_LITERAL(20, 257, 8), // "fullPath"
QT_MOC_LITERAL(21, 266, 6), // "hasLFT"
QT_MOC_LITERAL(22, 273, 20), // "hasLFTSubdirectories"
QT_MOC_LITERAL(23, 294, 7), // "addPath"
QT_MOC_LITERAL(24, 302, 24), // "index_files_in_directory"
QT_MOC_LITERAL(25, 327, 14), // "directory_path"
QT_MOC_LITERAL(26, 342, 14), // "delay_indexing"
QT_MOC_LITERAL(27, 357, 5), // "delay"
QT_MOC_LITERAL(28, 363, 15), // "cache_directory"
QT_MOC_LITERAL(29, 379, 8), // "maxCount"
QT_MOC_LITERAL(30, 388, 5), // "icase"
QT_MOC_LITERAL(31, 394, 11), // "startOffset"
QT_MOC_LITERAL(32, 406, 9), // "endOffset"
QT_MOC_LITERAL(33, 416, 7), // "keyword"
QT_MOC_LITERAL(34, 424, 9), // "useRegExp"
QT_MOC_LITERAL(35, 434, 8), // "quint32&"
QT_MOC_LITERAL(36, 443, 17), // "startOffsetReturn"
QT_MOC_LITERAL(37, 461, 15), // "endOffsetReturn"
QT_MOC_LITERAL(38, 477, 14), // "parallelsearch"
QT_MOC_LITERAL(39, 492, 5), // "rules"
QT_MOC_LITERAL(40, 498, 12), // "async_search"
QT_MOC_LITERAL(41, 511, 9) // "Q_NOREPLY"

    },
    "base_event_handler\0D-Bus Interface\0"
    "com.deepin.anything\0autoIndexInternalChanged\0"
    "\0autoIndexInternal\0autoIndexExternalChanged\0"
    "autoIndexExternal\0asyncSearchCompleted\0"
    "results\0search\0path\0keywords\0offset\0"
    "max_count\0type\0after\0before\0"
    "traverse_directory\0removePath\0fullPath\0"
    "hasLFT\0hasLFTSubdirectories\0addPath\0"
    "index_files_in_directory\0directory_path\0"
    "delay_indexing\0delay\0cache_directory\0"
    "maxCount\0icase\0startOffset\0endOffset\0"
    "keyword\0useRegExp\0quint32&\0startOffsetReturn\0"
    "endOffsetReturn\0parallelsearch\0rules\0"
    "async_search\0Q_NOREPLY"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_base_event_handler[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
      18,   16, // methods
       2,  204, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    1,  106,    4, 0x06 /* Public */,
       6,    1,  109,    4, 0x06 /* Public */,
       8,    1,  112,    4, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      10,    4,  115,    4, 0x0a /* Public */,
      10,    2,  124,    4, 0x0a /* Public */,
      10,    3,  129,    4, 0x0a /* Public */,
      10,    4,  136,    4, 0x0a /* Public */,
      18,    1,  145,    4, 0x0a /* Public */,
      19,    1,  148,    4, 0x0a /* Public */,
      21,    1,  151,    4, 0x0a /* Public */,
      22,    1,  154,    4, 0x0a /* Public */,
      23,    1,  157,    4, 0x0a /* Public */,
      24,    1,  160,    4, 0x0a /* Public */,
      26,    1,  163,    4, 0x0a /* Public */,
      28,    0,  166,    4, 0x0a /* Public */,
      10,    9,  167,    4, 0x0a /* Public */,
      38,    7,  186,    4, 0x0a /* Public */,
      40,    1,  201,   41, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::Bool,    5,
    QMetaType::Void, QMetaType::Bool,    7,
    QMetaType::Void, QMetaType::QStringList,    9,

 // slots: parameters
    QMetaType::QStringList, QMetaType::QString, QMetaType::QString, QMetaType::Int, QMetaType::Int,   11,   12,   13,   14,
    QMetaType::QStringList, QMetaType::QString, QMetaType::QString,   11,   12,
    QMetaType::QStringList, QMetaType::QString, QMetaType::QString, QMetaType::QString,   11,   12,   15,
    QMetaType::QStringList, QMetaType::QString, QMetaType::QString, QMetaType::QString, QMetaType::QString,   11,   12,   16,   17,
    QMetaType::QStringList, QMetaType::QString,   11,
    QMetaType::Bool, QMetaType::QString,   20,
    QMetaType::Bool, QMetaType::QString,   11,
    QMetaType::QStringList, QMetaType::QString,   11,
    QMetaType::Void, QMetaType::QString,   20,
    QMetaType::Void, QMetaType::QString,   25,
    QMetaType::Void, QMetaType::Bool,   27,
    QMetaType::QString,
    QMetaType::QStringList, QMetaType::Int, QMetaType::LongLong, QMetaType::UInt, QMetaType::UInt, QMetaType::QString, QMetaType::QString, QMetaType::Bool, 0x80000000 | 35, 0x80000000 | 35,   29,   30,   31,   32,   11,   33,   34,   36,   37,
    QMetaType::QStringList, QMetaType::QString, QMetaType::UInt, QMetaType::UInt, QMetaType::QString, QMetaType::QStringList, 0x80000000 | 35, 0x80000000 | 35,   11,   31,   32,   33,   39,   36,   37,
    QMetaType::Void, QMetaType::QString,   12,

 // properties: name, type, flags
       5, QMetaType::Bool, 0x00495103,
       7, QMetaType::Bool, 0x00495103,

 // properties: notify_signal_id
       0,
       1,

       0        // eod
};

void base_event_handler::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<base_event_handler *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->autoIndexInternalChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 1: _t->autoIndexExternalChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 2: _t->asyncSearchCompleted((*reinterpret_cast< const QStringList(*)>(_a[1]))); break;
        case 3: { QStringList _r = _t->search((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< QString(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3])),(*reinterpret_cast< int(*)>(_a[4])));
            if (_a[0]) *reinterpret_cast< QStringList*>(_a[0]) = std::move(_r); }  break;
        case 4: { QStringList _r = _t->search((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QStringList*>(_a[0]) = std::move(_r); }  break;
        case 5: { QStringList _r = _t->search((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< QString(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< QStringList*>(_a[0]) = std::move(_r); }  break;
        case 6: { QStringList _r = _t->search((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< QString(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])),(*reinterpret_cast< const QString(*)>(_a[4])));
            if (_a[0]) *reinterpret_cast< QStringList*>(_a[0]) = std::move(_r); }  break;
        case 7: { QStringList _r = _t->traverse_directory((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QStringList*>(_a[0]) = std::move(_r); }  break;
        case 8: { bool _r = _t->removePath((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 9: { bool _r = _t->hasLFT((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 10: { QStringList _r = _t->hasLFTSubdirectories((*reinterpret_cast< QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QStringList*>(_a[0]) = std::move(_r); }  break;
        case 11: _t->addPath((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 12: _t->index_files_in_directory((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 13: _t->delay_indexing((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 14: { QString _r = _t->cache_directory();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 15: { QStringList _r = _t->search((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< qint64(*)>(_a[2])),(*reinterpret_cast< quint32(*)>(_a[3])),(*reinterpret_cast< quint32(*)>(_a[4])),(*reinterpret_cast< const QString(*)>(_a[5])),(*reinterpret_cast< QString(*)>(_a[6])),(*reinterpret_cast< bool(*)>(_a[7])),(*reinterpret_cast< quint32(*)>(_a[8])),(*reinterpret_cast< quint32(*)>(_a[9])));
            if (_a[0]) *reinterpret_cast< QStringList*>(_a[0]) = std::move(_r); }  break;
        case 16: { QStringList _r = _t->parallelsearch((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< quint32(*)>(_a[2])),(*reinterpret_cast< quint32(*)>(_a[3])),(*reinterpret_cast< const QString(*)>(_a[4])),(*reinterpret_cast< const QStringList(*)>(_a[5])),(*reinterpret_cast< quint32(*)>(_a[6])),(*reinterpret_cast< quint32(*)>(_a[7])));
            if (_a[0]) *reinterpret_cast< QStringList*>(_a[0]) = std::move(_r); }  break;
        case 17: _t->async_search((*reinterpret_cast< QString(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (base_event_handler::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&base_event_handler::autoIndexInternalChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (base_event_handler::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&base_event_handler::autoIndexExternalChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (base_event_handler::*)(const QStringList & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&base_event_handler::asyncSearchCompleted)) {
                *result = 2;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<base_event_handler *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< bool*>(_v) = _t->autoIndexInternal(); break;
        case 1: *reinterpret_cast< bool*>(_v) = _t->autoIndexExternal(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<base_event_handler *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setAutoIndexInternal(*reinterpret_cast< bool*>(_v)); break;
        case 1: _t->setAutoIndexExternal(*reinterpret_cast< bool*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject base_event_handler::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_base_event_handler.data,
    qt_meta_data_base_event_handler,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *base_event_handler::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *base_event_handler::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_base_event_handler.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int base_event_handler::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 18)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 18;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 2;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void base_event_handler::autoIndexInternalChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void base_event_handler::autoIndexExternalChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void base_event_handler::asyncSearchCompleted(const QStringList & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
