/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Command line was: qdbusxml2cpp -m -a anythingadaptor -i /build/deepin-anything/src/deepin-anything/src/server/include/core/base_event_handler.h -l base_event_handler /build/deepin-anything/src/build/src/server/com.deepin.Anything.xml
 *
 * qdbusxml2cpp is Copyright (C) 2023 The Qt Company Ltd.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "anythingadaptor.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class AnythingAdaptor
 */

AnythingAdaptor::AnythingAdaptor(base_event_handler *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

AnythingAdaptor::~AnythingAdaptor()
{
    // destructor
}

bool AnythingAdaptor::autoIndexExternal() const
{
    // get the value of property autoIndexExternal
    return qvariant_cast< bool >(parent()->property("autoIndexExternal"));
}

void AnythingAdaptor::setAutoIndexExternal(bool value)
{
    // set the value of property autoIndexExternal
    parent()->setProperty("autoIndexExternal", QVariant::fromValue(value));
}

bool AnythingAdaptor::autoIndexInternal() const
{
    // get the value of property autoIndexInternal
    return qvariant_cast< bool >(parent()->property("autoIndexInternal"));
}

void AnythingAdaptor::setAutoIndexInternal(bool value)
{
    // set the value of property autoIndexInternal
    parent()->setProperty("autoIndexInternal", QVariant::fromValue(value));
}

void AnythingAdaptor::addPath(const QString &fullPath)
{
    // handle method call com.deepin.anything.addPath
    parent()->addPath(fullPath);
}

void AnythingAdaptor::async_search(const QString &keywords)
{
    // handle method call com.deepin.anything.async_search
    parent()->async_search(keywords);
}

QString AnythingAdaptor::cache_directory()
{
    // handle method call com.deepin.anything.cache_directory
    return parent()->cache_directory();
}

void AnythingAdaptor::delay_indexing(bool delay)
{
    // handle method call com.deepin.anything.delay_indexing
    parent()->delay_indexing(delay);
}

bool AnythingAdaptor::hasLFT(const QString &path)
{
    // handle method call com.deepin.anything.hasLFT
    return parent()->hasLFT(path);
}

QStringList AnythingAdaptor::hasLFTSubdirectories(const QString &path)
{
    // handle method call com.deepin.anything.hasLFTSubdirectories
    return parent()->hasLFTSubdirectories(path);
}

void AnythingAdaptor::index_files_in_directory(const QString &directory_path)
{
    // handle method call com.deepin.anything.index_files_in_directory
    parent()->index_files_in_directory(directory_path);
}

QStringList AnythingAdaptor::parallelsearch(const QString &path, uint startOffset, uint endOffset, const QString &keyword, const QStringList &rules, uint &startOffsetReturn, uint &endOffsetReturn)
{
    // handle method call com.deepin.anything.parallelsearch
    return parent()->parallelsearch(path, startOffset, endOffset, keyword, rules, startOffsetReturn, endOffsetReturn);
}

bool AnythingAdaptor::removePath(const QString &fullPath)
{
    // handle method call com.deepin.anything.removePath
    return parent()->removePath(fullPath);
}

QStringList AnythingAdaptor::search(int maxCount, qlonglong icase, uint startOffset, uint endOffset, const QString &path, const QString &keyword, bool useRegExp, uint &startOffsetReturn, uint &endOffsetReturn)
{
    // handle method call com.deepin.anything.search
    return parent()->search(maxCount, icase, startOffset, endOffset, path, keyword, useRegExp, startOffsetReturn, endOffsetReturn);
}

QStringList AnythingAdaptor::search(const QString &path, const QString &keywords, const QString &after, const QString &before)
{
    // handle method call com.deepin.anything.search
    return parent()->search(path, keywords, after, before);
}

QStringList AnythingAdaptor::search(const QString &path, const QString &keywords, const QString &type)
{
    // handle method call com.deepin.anything.search
    return parent()->search(path, keywords, type);
}

QStringList AnythingAdaptor::search(const QString &path, const QString &keywords)
{
    // handle method call com.deepin.anything.search
    return parent()->search(path, keywords);
}

QStringList AnythingAdaptor::search(const QString &path, const QString &keywords, int offset, int max_count)
{
    // handle method call com.deepin.anything.search
    return parent()->search(path, keywords, offset, max_count);
}

QStringList AnythingAdaptor::traverse_directory(const QString &path)
{
    // handle method call com.deepin.anything.traverse_directory
    return parent()->traverse_directory(path);
}


#include "anythingadaptor.moc"
